; Routine for reading in data from a card reader
;
; CARD_DATA, CARD_PRESENCE, CARD_CLK are assumed to be defined
; to the pins that the reader is connected to.

#ifndef CARD_SCAN_ASM
#define CARD_SCAN_ASM


	SEGU data
	
card_read_temp DS 1
card_read_temp2 DS 1
card_read_code_count DS 1
card_read_bit_count DS 1
card_read_data DS 1
card_read_cksum DS 1
	
	SEG code

;---------------
; READ_CARD
;
; Attempts to read a card in the card reader... it waits for the presence to go
; low, and then waits for a sentinel start.  After which it reads 5 bit encoded
; data, and checks parity.  If anything doesnt match up it returns nonzero, else it
; puts the 20 bytes of data into FSR and returns 0.


READ_CARD
	btfsc	CARD_PRESENCE	; wait for the card presence line
	goto	READ_CARD		; to go low....

	clrf	card_read_cksum
	movlw	20
	movwf	card_read_code_count
	
	; now we clock in bits until we get a sentinel code
	movlw	8
	movwf	card_read_bit_count
	clrf	card_read_data
	
sentinel_search
	btfsc	CARD_PRESENCE
	retlw	1

	btfss	CARD_CLK
	goto	sentinel_search

ss_wait_falling_edge	
	btfsc	CARD_PRESENCE
	retlw	2
	
	btfsc	CARD_CLK
	goto	ss_wait_falling_edge

	bcf		STATUS,C
	btfss	CARD_DATA
	bsf		STATUS,C
	rlf		card_read_data,F
	
	
	movf	card_read_data,W
	sublw	B'11010'		; the sentinel code
	
	btfss	STATUS,Z
	goto	sentinel_search	; if this isnt the code, then wait for next bit


; so now weve received the sentinel code... shift in the 5 bit codes, check
; the parity, and store away in the locations pointed by FSR

	movlw	0
	movwf	card_read_temp

read_5_bits
	movlw	5
	movwf	card_read_bit_count
	clrf	card_read_data
	
r5b_wait_for_clock_high
	btfsc	CARD_PRESENCE
	retlw	3

	btfss	CARD_CLK
	goto	r5b_wait_for_clock_high

r5b_wait_for_clock_low
	btfsc	CARD_PRESENCE
	retlw	4
	
	btfsc	CARD_CLK
	goto	r5b_wait_for_clock_low

	bcf		STATUS,C
	btfss	CARD_DATA
	bsf		STATUS,C
	rrf		card_read_data,F
	decfsz	card_read_bit_count,F
	goto	r5b_wait_for_clock_high
	
	; here we should check the parity
	movf	card_read_data,W
	movwf	card_read_temp2
	swapf	card_read_temp2,W
	xorwf	card_read_temp2,W
	movwf	card_read_temp2
	rlf		card_read_temp2,F
	rlf		card_read_temp2,F
	xorwf	card_read_temp2,W
	movwf	card_read_temp2
	rlf		card_read_temp2,F
	xorwf	card_read_temp2,F
	btfss	card_read_temp2,7
	retlw	6	; parity check failed

	; now we test for the end sentinel code
	movf	card_read_data,W
	sublw	B'11111000'
	btfss	STATUS,Z
	goto	not_end_sentinel	; if we got the end sentinel, then we are done

	movlw	1
	movwf	card_read_code_count
	clrf	card_read_temp

not_end_sentinel
	; if the low bit of card_read_temp is 0, then we put this guy in the upper
	; 4 bits of IND0, otherwise we put him in the lower 4 bits and inc FSR and dec byte
	; count
	
	bcf		STATUS,C
	rlf		card_read_data,F 	; shift away the parity bit

	; after the parity bit has been shifted away, we should check to see if this is
	; a control character or not, if it is a regular data character we need to
	; add it to the current checksum
	
	movf	card_read_data,W
	sublw	B'10010000'
	btfss	STATUS,C
	goto	not_data_char
	movf	card_read_data,W
	addwf	card_read_cksum,F
	
not_data_char

	incf	card_read_temp,F

	btfss	card_read_temp,0
	goto	r5b_low_nibble
	
	; so we are going to put it in the high nibble of IND0

	
	movlw	0x0F
	andwf	IND0,F
	movf	card_read_data,W
	iorwf	IND0,F
	goto	read_5_bits
	
r5b_low_nibble
	; so here we put it in the low nibble, and dec the byte counter to see
	; if were done
	
	movlw	0xF0
	andwf	IND0,F
	
	swapf	card_read_data,W
	iorwf	IND0,F
	
	incf	FSR,F
	
	decfsz	card_read_code_count,F
	goto	read_5_bits
	

wait_for_done
	; wait for the presence line to go back high
	btfss	CARD_PRESENCE
	goto	wait_for_done
	
	decf	FSR,F
	; need to verify checksum... this is in reverse bit order now...
	clrf	card_read_temp
	
	rlf		card_read_cksum,F
	rrf		card_read_temp,F
	rlf		card_read_cksum,F
	rrf		card_read_temp,F
	rlf		card_read_cksum,F
	rrf		card_read_temp,F
	rlf		card_read_cksum,F
	rrf		card_read_temp,F
	swapf	card_read_temp,F
	movf	IND0,W
	andlw	0x0f
	subwf	card_read_temp,F
	btfsc	STATUS,Z
	retlw	8
	
	swapf	card_read_temp,W
	
		
	
	retlw	0	

#endif

