; Here are some routines for talking to a 4x20 HD44780 LCD
; display.
;
; The following pinouts are assumed:
;
;                        -----------
;                       |     U     |
;               HD_E  --|RA2     RA1|-- HD_R/W
;                     --|RA3     RA0|-- HD_RS 
;                     --|RA4        |
;                       |           |
;                       |           |
;                       |           |
;                     --|RB0     RB7|-- D7
;                     --|RB1     RB6|-- D6
;                     --|RB2     RB5|-- D5
;                     --|RB3     RB4|-- D4
;                       |           |
;                        ----------- 
;
; The control lines can be defined to whichever pin works best, however the
; data lines must be at the B4-7 pins unless you wish to modify the
; PUTCHAR/PUTCMD routines.
;
; Code Space: 124 bytes
; Data Mem: 3 bytes

#ifndef LCD_ASM
#define LCD_ASM

	SEGU data

lcd_temp DS 1
numdisp_temp DS 1
table_index DS 1

	SEG code

Print_Label
    movwf   table_index
    call    LabelTable
    xorlw   0
    btfsc   STATUS,Z
    goto    Print_End
    call    LCD_PUTCHAR
    movf    table_index,W
    addlw   1  
    goto    Print_Label   
Print_End
    return

; writes the value in W to the display in hex with a period inbetween
; the two digits
LCD_DISPNUMPERIOD
    movwf   numdisp_temp
    swapf   numdisp_temp,W
    call    Num_Table
    call    LCD_PUTCHAR
    movlw   '.'
    call    LCD_PUTCHAR
    movf    numdisp_temp,W
    call    Num_Table
    call    LCD_PUTCHAR
    return

; writes the value in W to the display in hex
LCD_DISPNUM
    movwf   numdisp_temp
    swapf   numdisp_temp,W
    call    Num_Table
    call    LCD_PUTCHAR
    movf    numdisp_temp,W
    call    Num_Table
    call    LCD_PUTCHAR
    return

; this routine mangles the contents of other PORTB pins

init_lcd
	bcf		HD_E
	bcf		HD_RS
	bcf		HD_RW
	movf	PORTB,W
	andlw	0x0f
	movwf	PORTB

    movlw   0x28
    call LCD_PUTCMD

    movlw   0x00
    call LCD_DMODE
    call LCD_CLEAR
    movlw   0x04  
    call    LCD_DMODE
    movlw   0x02
    call    LCD_EMODE
    
    return

LCD_SDDAM
    iorlw   0x80
    call    LCD_PUTCMD
    return

LCD_SCGAM
    andlw   0x3f
    iorlw   0x40
    call    LCD_PUTCMD

LCD_CLEAR
    movlw   0x01
    call    LCD_PUTCMD

LCD_EMODE
    andlw   0x03
    iorlw   0x04
    call    LCD_PUTCMD
    return

LCD_DMODE
    andlw   0x07
    iorlw   0x08
    call    LCD_PUTCMD
    return

LCD_BUSY
    movlw   0x0f
    andwf   PORTB,F
    bsf STATUS,RP0 
    movlw   B'11110000'
    iorwf   TRISB,F
    bcf STATUS,RP0 
    bcf HD_RS
    bsf HD_RW
    bsf HD_E 
    movf    PORTB,W
    bcf HD_E
    andlw 0xf0
    movwf   temp
    bsf HD_E
    bcf HD_E
    swapf   PORTB,W
    andlw 0x0f
    iorwf   temp,F
    btfsc   temp, 7 
    goto    LCD_BUSY
    bsf STATUS,RP0  
    movlw   B'00001111'
    andwf   TRISB,F
    bcf STATUS,RP0 
    return

; put the command in A
LCD_PUTCMD
    movwf   lcd_temp
    call    LCD_BUSY
    movlw   0x0f
    andwf   PORTB,F
    movf    lcd_temp, W
    andlw   0xf0
    iorwf   PORTB,F
    bcf HD_RW
    bcf HD_RS
    bsf HD_E 
    bcf HD_E 
    movlw   0x0f
    andwf   PORTB,F
    swapf   lcd_temp, W
    andlw   0xf0
    iorwf   PORTB,F
    bsf HD_E
    bcf HD_E
    return  

LCD_PUTCHAR
    movwf   lcd_temp
    call    LCD_BUSY
    movlw   0x0f
    andwf   PORTB,F
    movf    lcd_temp,W
    andlw   0xf0
    iorwf   PORTB,F
    bcf HD_RW
    bsf HD_RS
    bsf HD_E 
    bcf HD_E 
    movlw   0x0f
    andwf   PORTB,F
    swapf   lcd_temp, W
    andlw   0xf0
    iorwf   PORTB,F
    bsf HD_RS
    bsf HD_E 
    bcf HD_E 
    return   

#endif
