; Here are some routines for talking to a 4x20 HD44780 LCD
; over my 2 wire interface.
;
; TWOWIRE_CLK is to be defined as the clock port and pin
; and TWOWIRE_DATA is to be defined as the data port and pin

#ifndef LCD_2WIRE_ASM
#define LCD_2WIRE_ASM

#include "../include/baud_rate_calcs.h"
#include "../include/uart_delay.asm"

	SEGU data

lcd_temp 	DS 1
numdisp_temp DS 1
table_index DS 1
twowire_len DS 1
twowire_data DS 2
twowire_buf DS 1
twowire_data_index DS 1
twowire_bit_count DS 1

	SEG code

Print_Label
    movwf   table_index
    call    LabelTable 
    xorlw   0
    btfsc   STATUS,Z
    goto    Print_End
    call    LCD_PUTCHAR
    movf    table_index,W
    addlw   1
    goto    Print_Label
Print_End
init_lcd
    return

; writes the value in W to the display in hex
LCD_DISPNUM
    movwf   numdisp_temp
    swapf   numdisp_temp,W
    call    Num_Table
    call    LCD_PUTCHAR
    movf    numdisp_temp,W
    call    Num_Table
    call    LCD_PUTCHAR
    return

; writes the value in W to the display in hex
LCD_DISPNUMPERIOD
    movwf   numdisp_temp
    swapf   numdisp_temp,W
    call    Num_Table
    call    LCD_PUTCHAR
    movlw	'.'
    call	LCD_PUTCHAR
    movf    numdisp_temp,W
    call    Num_Table
    call    LCD_PUTCHAR
    return


LCD_SDDAM
    iorlw   0x80
    call    LCD_PUTCMD
    return

LCD_CLEAR
    movlw   0x01
    call    LCD_PUTCMD

LCD_EMODE
    andlw   0x03
    iorlw   0x04
    call    LCD_PUTCMD
    return

LCD_DMODE
    andlw   0x07
    iorlw   0x08
    call    LCD_PUTCMD
    return

; put the command in A
LCD_PUTCMD
    movwf   twowire_data+1
    clrf    twowire_data  
    
    movlw   2   ; we are going to send two bytes
    call    twowire_send
    
    return

LCD_PUTCHAR
    movwf   twowire_data
    
    movlw   1
    call    twowire_send
    return

twowire_send
    movwf   twowire_len
    ; first well send the start indicator
    bsf TWOWIRE_DATA
    nop
    bsf TWOWIRE_CLK
    movlw   3
    call    delaybit
    bcf TWOWIRE_DATA
    nop
    bcf TWOWIRE_CLK
    
    movlw   7
    call    delaybit
    
    movlw   twowire_data
    movwf   FSR
    
twowire_send_loop
    movf    IND0,W
    movwf   twowire_buf
    call    twowire_send_byte
    incf    FSR,F
    movlw   2
    call    delaybit
    decfsz  twowire_len,F
    goto    twowire_send_loop

    ; now we send the stop bit

    bcf TWOWIRE_DATA
    nop
    bsf TWOWIRE_CLK
    movlw   3
    call    delaybit
    bsf TWOWIRE_DATA
    nop
    bcf TWOWIRE_CLK
    movlw   7
    call    delaybit
    return

twowire_send_byte
    movlw   8
    movwf   twowire_bit_count
twowire_sb_loop
    rlf twowire_buf,F
    btfss   STATUS,C 
    bcf TWOWIRE_DATA 
    btfsc   STATUS,C 
    bsf TWOWIRE_DATA 
    nop
    bsf TWOWIRE_CLK
    movlw   2
    call    delaybit
    bcf TWOWIRE_CLK 
    movlw   2
    call    delaybit
    decfsz  twowire_bit_count,F
    goto    twowire_sb_loop
    
    return

#endif
