#ifndef UART_3110
#define UART_3110

#include "../include/spi.asm"

; Interface to MAXIM 3110 UART using my SPI interface
;
; the CS pin is assumed to be MAX3310_CS

; The following are the configuration bits that can be set

#define B9600 0x0b
#define B19200 0x0a
#define B38400 0x09
#define B76800 0x08
#define B57600 0x02
#define B115200 0x01

#define FIFO_DISABLED 0x2000
#define SOFT_SHUTDOWN 0x1000
#define TBUF_INT_ENABLED 0x0800
#define FIFO_INT_ENABLED 0x0400
#define PARITY_INT_ENABLED 0x0200
#define FRAMING_ERROR_INT_ENABLED 0x0100
#define IRDA_ENABLED 0x0080
#define TWO_STOP_BITS 0x0040
#define PARITY_ENABLED 0x0020
#define SEVEN_BIT_WORDS 0x0010


#define WRITE_CONFIGURATION 0xc000
#define READ_CONFIGURATION 0x4000
#define WRITE_DATA 0x8000
#define READ_DATA 0x0000

#define UART_3110_CONFIG WRITE_CONFIGURATION | BAUD_RATE_3110

	SEGU data
	
uart_3110_temp DS 1	
uart_rx_buf DS 1
	
	SEG code


;----------------
; UART_INIT
;
; Called to initialize the UART
;

UART_232_INIT
	bsf		MAX3110_CS
	movlw	B'00001100'
	TRIS	6
	movlw	HIGH(UART_3110_CONFIG)
	movwf	spi_high
	movlw	LOW(UART_3110_CONFIG)
	movwf	spi_low
	bcf		MAX3110_CS
	call	SPI_READWRITE16
	bsf		MAX3110_CS
	
	movlw	HIGH(READ_CONFIGURATION)
	movwf	spi_high
	clrf	spi_low
	
	bcf		MAX3110_CS

	call	SPI_READWRITE16
	bsf		MAX3110_CS
	movlw	B'00001110'
	TRIS	6
	
	movf	spi_low,W
	sublw	LOW(UART_3110_CONFIG)
	btfss	STATUS,Z
	goto	UART_232_INIT	
	
	retlw 0

;----------------
; UART_RX
;
; receives a byte into uart_rx_buf
; if no bytes are available 0 is returned in W otherwise
; 1 is returned...

UART_232_RX
	bsf		MAX3110_CS
	movlw	B'00001100'
	TRIS	6
	movlw	HIGH(READ_DATA)
	movwf	spi_high
	movlw	LOW(READ_DATA)
	movwf	spi_low

	bcf		MAX3110_CS
	call	SPI_READWRITE16
	bsf		MAX3110_CS
	movlw	B'00001110'
	TRIS	6

	btfss	spi_high,7
	retlw	0

	movf	spi_low,W
	movwf	uart_rx_buf
	retlw	1
	
;----------------
; UART_TX
;
; the byte to send is in W
;

UART_232_TX
	; first have to wait to make sure the uart has room
	; for our character....
	movwf	uart_3110_temp
	
uart_232_tx_wait
	bsf		MAX3110_CS
	movlw	B'00001100'
	TRIS	6

	movlw	HIGH(READ_CONFIGURATION)
	movwf	spi_high
	clrf	spi_low

	bcf		MAX3110_CS
	call	SPI_READWRITE16

	bsf		MAX3110_CS
	
	btfss	spi_high,6
	goto	uart_232_tx_wait
	
	movf	uart_3110_temp,W
	movwf	spi_low
	movlw	HIGH(WRITE_DATA)
	movwf	spi_high

	bcf		MAX3110_CS
	call	SPI_READWRITE16
	bsf		MAX3110_CS
	movlw	B'00001110'
	TRIS	6

	retlw 0


#endif
